@@ -18,4 +18,12 @@ list_exceptions = [FileExistsError, FileNotFoundError]
 try:
     pass
 except (*list_exceptions,):
-    pass
\ No newline at end of file
+    pass
+
+try:
+    ...
+except (
+        ValueError,
+        # text
+        ):
+    ...
\ No newline at end of file
