@@ -51,3 +51,13 @@ def test_error_match_is_empty():
 def test_ok_t_string_match():
     with pytest.raises(ValueError, match=t""):
         raise ValueError("Can't divide 1 by 0")
+
+
+def test_ok_check_provided():
+    with pytest.raises(ValueError, check=lambda e: str(e) == "error"):
+        raise ValueError("error")
+
+
+def test_error_check_is_none():
+    with pytest.raises(ValueError, check=None):
+        raise ValueError("Can't divide 1 by 0")
