@@ -38,3 +38,13 @@ for country, stars in(zip)(flag_stars.keys(), flag_stars.values()):...
 # Regression test for https://github.com/astral-sh/ruff/issues/18778
 d = {}
 for country, stars in zip(d.keys(*x), d.values("hello")):...
+
+
+flag_stars = {"USA": 50, "Slovenia": 3, "Panama": 2, "Australia": 6}
+
+for country, stars in zip(
+        flag_stars.keys(),
+        # text
+        flag_stars.values(),
+):
+    print(f"{country}'s flag has {stars} stars.")
