@@ -296,3 +296,13 @@ pub(crate) const fn is_s310_resolve_string_literal_bindings_enabled(
 pub(crate) const fn is_range_suppressions_enabled(settings: &LinterSettings) -> bool {
     settings.preview.is_enabled()
 }
+
+// https://github.com/astral-sh/ruff/pull/22057
+pub(crate) const fn is_ble001_exc_info_suppression_enabled(settings: &LinterSettings) -> bool {
+    settings.preview.is_enabled()
+}
+
+// https://github.com/astral-sh/ruff/pull/22419
+pub(crate) const fn is_py315_support_enabled(settings: &LinterSettings) -> bool {
+    settings.preview.is_enabled()
+}
