@@ -26,12 +26,13 @@ use ruff_source_file::SourceFileBuilder;
 use crate::codes::Rule;
 use crate::fix::{FixResult, fix_file};
 use crate::linter::check_path;
-use crate::message::{EmitterContext, create_syntax_error_diagnostic};
+use crate::message::EmitterContext;
 use crate::package::PackageRoot;
 use crate::packaging::detect_package_root;
 use crate::settings::types::UnsafeFixes;
 use crate::settings::{LinterSettings, flags};
 use crate::source_kind::SourceKind;
+use crate::suppression::Suppressions;
 use crate::{Applicability, FixAvailability};
 use crate::{Locator, directives};
 
@@ -234,6 +235,8 @@ pub(crate) fn test_contents<'a>(
         &locator,
         &indexer,
     );
+    let suppressions =
+        Suppressions::from_tokens(settings, locator.contents(), parsed.tokens(), &indexer);
     let messages = check_path(
         path,
         path.parent()
@@ -249,6 +252,7 @@ pub(crate) fn test_contents<'a>(
         source_type,
         &parsed,
         target_version,
+        &suppressions,
     );
 
     let source_has_errors = parsed.has_invalid_syntax();
@@ -299,6 +303,8 @@ pub(crate) fn test_contents<'a>(
                 &indexer,
             );
 
+            let suppressions =
+                Suppressions::from_tokens(settings, locator.contents(), parsed.tokens(), &indexer);
             let fixed_messages = check_path(
                 path,
                 None,
@@ -312,6 +318,7 @@ pub(crate) fn test_contents<'a>(
                 source_type,
                 &parsed,
                 target_version,
+                &suppressions,
             );
 
             if parsed.has_invalid_syntax() && !source_has_errors {
@@ -405,7 +412,7 @@ Either ensure you always emit a fix or change `Violation::FIX_AVAILABILITY` to e
             diagnostic
         })
         .chain(parsed.errors().iter().map(|parse_error| {
-            create_syntax_error_diagnostic(source_code.clone(), &parse_error.error, parse_error)
+            Diagnostic::invalid_syntax(source_code.clone(), &parse_error.error, parse_error)
         }))
         .sorted_by(Diagnostic::ruff_start_ordering)
         .collect();
@@ -419,7 +426,7 @@ fn print_syntax_errors(errors: &[ParseError], path: &Path, source: &SourceKind)
     let messages: Vec<_> = errors
         .iter()
         .map(|parse_error| {
-            create_syntax_error_diagnostic(source_file.clone(), &parse_error.error, parse_error)
+            Diagnostic::invalid_syntax(source_file.clone(), &parse_error.error, parse_error)
         })
         .collect();
 
@@ -508,12 +515,18 @@ macro_rules! assert_diagnostics {
 
 #[macro_export]
 macro_rules! assert_diagnostics_diff {
-    ($snapshot:expr, $path:expr, $settings_before:expr, $settings_after:expr) => {{
+    ($snapshot:expr, $path:expr, $settings_before:expr, $settings_after:expr $(,)?) => {{
         let diff = $crate::test::test_path_with_settings_diff($path, $settings_before, $settings_after)?;
         insta::with_settings!({ omit_expression => true }, {
             insta::assert_snapshot!($snapshot, format!("{}", diff));
         });
     }};
+    ($path:expr, $settings_before:expr, $settings_after:expr $(,)?) => {{
+        let diff = $crate::test::test_path_with_settings_diff($path, $settings_before, $settings_after)?;
+        insta::with_settings!({ omit_expression => true }, {
+            insta::assert_snapshot!(format!("{}", diff));
+        });
+    }};
 }
 
 #[cfg(test)]
