@@ -1,19 +1,23 @@
-use crate::{FormatNodeRule, FormattedIterExt, PyFormatter};
-use ruff_formatter::prelude::{format_args, format_with, space, text};
-use ruff_formatter::{write, Buffer, FormatResult};
+use ruff_formatter::{format_args, write};
 use ruff_python_ast::StmtImport;
 
+use crate::prelude::*;
+
 #[derive(Default)]
 pub struct FormatStmtImport;
 
 impl FormatNodeRule<StmtImport> for FormatStmtImport {
     fn fmt_fields(&self, item: &StmtImport, f: &mut PyFormatter) -> FormatResult<()> {
-        let StmtImport { names, range: _ } = item;
+        let StmtImport {
+            names,
+            range: _,
+            node_index: _,
+        } = item;
         let names = format_with(|f| {
-            f.join_with(&format_args![text(","), space()])
+            f.join_with(&format_args![token(","), space()])
                 .entries(names.iter().formatted())
                 .finish()
         });
-        write!(f, [text("import"), space(), names])
+        write!(f, [token("import"), space(), names])
     }
 }
