@@ -4,19 +4,25 @@ use ruff_formatter::{
 use ruff_python_ast::helpers::is_compound_statement;
 use ruff_python_ast::{self as ast, Expr, PySourceType, Stmt, Suite};
 use ruff_python_ast::{AnyNodeRef, StmtExpr};
-use ruff_python_trivia::{lines_after, lines_after_ignoring_end_of_line_trivia, lines_before};
+use ruff_python_trivia::{
+    SimpleTokenKind, SimpleTokenizer, lines_after, lines_after_ignoring_end_of_line_trivia,
+    lines_before,
+};
 use ruff_text_size::{Ranged, TextRange};
 
 use crate::comments::{
-    Comments, LeadingDanglingTrailingComments, leading_comments, trailing_comments,
+    Comments, LeadingDanglingTrailingComments, has_skip_comment, leading_comments,
+    trailing_comments,
 };
 use crate::context::{NodeLevel, TopLevelStatementPosition, WithIndentLevel, WithNodeLevel};
 use crate::other::string_literal::StringLiteralKind;
 use crate::prelude::*;
-use crate::preview::is_blank_line_before_decorated_class_in_stub_enabled;
-use crate::statement::stmt_expr::FormatStmtExpr;
+use crate::preview::{
+    is_allow_newline_after_block_open_enabled, is_blank_line_before_decorated_class_in_stub_enabled,
+};
+use crate::statement::trailing_semicolon;
 use crate::verbatim::{
-    suppressed_node, write_suppressed_statements_starting_with_leading_comment,
+    write_skipped_statements, write_suppressed_statements_starting_with_leading_comment,
     write_suppressed_statements_starting_with_trailing_comment,
 };
 
@@ -150,7 +156,21 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
 
         let first_comments = comments.leading_dangling_trailing(first);
 
-        let (mut preceding, mut empty_line_after_docstring) = if first_comments
+        let (mut preceding, mut empty_line_after_docstring) = if let Some(verbatim_range) =
+            skip_range(first.statement(), iter.as_slice(), f.context())
+        {
+            let preceding =
+                write_skipped_statements(first.statement(), &mut iter, verbatim_range, f)?;
+
+            // Insert a newline after a module level docstring, but treat
+            // it as a docstring otherwise. See: https://github.com/psf/black/pull/3932.
+            let empty_line_after_docstring =
+                matches!(self.kind, SuiteKind::TopLevel | SuiteKind::Class)
+                    && DocstringStmt::try_from_statement(preceding, self.kind, f.context())
+                        .is_some();
+
+            (preceding, empty_line_after_docstring)
+        } else if first_comments
             .leading
             .iter()
             .any(|comment| comment.is_suppression_off_comment(source))
@@ -169,6 +189,22 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                 false,
             )
         } else {
+            // Allow an empty line after a function header in preview, if the function has no
+            // docstring and no initial comment.
+            let allow_newline_after_block_open =
+                is_allow_newline_after_block_open_enabled(f.context())
+                    && matches!(self.kind, SuiteKind::Function)
+                    && matches!(first, SuiteChildStatement::Other(_));
+
+            let start = comments
+                .leading(first)
+                .first()
+                .map_or_else(|| first.start(), Ranged::start);
+
+            if allow_newline_after_block_open && lines_before(start, f.context().source()) > 1 {
+                empty_line().fmt(f)?;
+            }
+
             first.fmt(f)?;
 
             let empty_line_after_docstring = if matches!(first, SuiteChildStatement::Docstring(_))
@@ -218,7 +254,7 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                     )?;
                 } else {
                     // Preserve empty lines after a stub implementation but don't insert a new one if there isn't any present in the source.
-                    // This is useful when having multiple function overloads that should be grouped to getter by omitting new lines between them.
+                    // This is useful when having multiple function overloads that should be grouped together by omitting new lines between them.
                     let is_preceding_stub_function_without_empty_line = following
                         .is_function_def_stmt()
                         && preceding
@@ -373,7 +409,10 @@ impl FormatRule<Suite, PyFormatContext<'_>> for FormatSuite {
                 }
             }
 
-            if following_comments
+            if let Some(verbatim_range) = skip_range(following, iter.as_slice(), f.context()) {
+                preceding = write_skipped_statements(following, &mut iter, verbatim_range, f)?;
+                preceding_comments = comments.leading_dangling_trailing(preceding);
+            } else if following_comments
                 .leading
                 .iter()
                 .any(|comment| comment.is_suppression_off_comment(source))
@@ -728,17 +767,21 @@ fn stub_suite_can_omit_empty_line(preceding: &Stmt, following: &Stmt, f: &PyForm
 
 /// Returns `true` if a function or class body contains only an ellipsis with no comments.
 pub(crate) fn contains_only_an_ellipsis(body: &[Stmt], comments: &Comments) -> bool {
-    match body {
-        [Stmt::Expr(ast::StmtExpr { value, .. })] => {
-            let [node] = body else {
-                return false;
-            };
-            value.is_ellipsis_literal_expr()
-                && !comments.has_leading(node)
-                && !comments.has_trailing_own_line(node)
-        }
-        _ => false,
+    as_only_an_ellipsis(body, comments).is_some()
+}
+
+/// Returns `Some(Stmt::Ellipsis)` if a function or class body contains only an ellipsis with no
+/// comments.
+pub(crate) fn as_only_an_ellipsis<'a>(body: &'a [Stmt], comments: &Comments) -> Option<&'a Stmt> {
+    if let [node @ Stmt::Expr(ast::StmtExpr { value, .. })] = body
+        && value.is_ellipsis_literal_expr()
+        && !comments.has_leading(node)
+        && !comments.has_trailing_own_line(node)
+    {
+        return Some(node);
     }
+
+    None
 }
 
 /// Returns `true` if a [`Stmt`] is a class or function definition.
@@ -818,61 +861,57 @@ impl Format<PyFormatContext<'_>> for DocstringStmt<'_> {
         let comments = f.context().comments().clone();
         let node_comments = comments.leading_dangling_trailing(self.docstring);
 
-        if FormatStmtExpr.is_suppressed(node_comments.trailing, f.context()) {
-            suppressed_node(self.docstring).fmt(f)
-        } else {
-            // SAFETY: Safe because `DocStringStmt` guarantees that it only ever wraps a `ExprStmt` containing a `ExprStringLiteral`.
-            let string_literal = self
-                .docstring
-                .as_expr_stmt()
-                .unwrap()
-                .value
-                .as_string_literal_expr()
-                .unwrap();
+        // SAFETY: Safe because `DocStringStmt` guarantees that it only ever wraps a `ExprStmt` containing a `ExprStringLiteral`.
+        let string_literal = self
+            .docstring
+            .as_expr_stmt()
+            .unwrap()
+            .value
+            .as_string_literal_expr()
+            .unwrap();
 
-            // We format the expression, but the statement carries the comments
-            write!(
-                f,
-                [
-                    leading_comments(node_comments.leading),
-                    f.options()
-                        .source_map_generation()
-                        .is_enabled()
-                        .then_some(source_position(self.docstring.start())),
-                    string_literal
-                        .format()
-                        .with_options(StringLiteralKind::Docstring),
-                    f.options()
-                        .source_map_generation()
-                        .is_enabled()
-                        .then_some(source_position(self.docstring.end())),
-                ]
-            )?;
+        // We format the expression, but the statement carries the comments
+        write!(
+            f,
+            [
+                leading_comments(node_comments.leading),
+                f.options()
+                    .source_map_generation()
+                    .is_enabled()
+                    .then_some(source_position(self.docstring.start())),
+                string_literal
+                    .format()
+                    .with_options(StringLiteralKind::Docstring),
+                f.options()
+                    .source_map_generation()
+                    .is_enabled()
+                    .then_some(source_position(self.docstring.end())),
+            ]
+        )?;
 
-            if self.suite_kind == SuiteKind::Class {
-                // Comments after class docstrings need a newline between the docstring and the
-                // comment (https://github.com/astral-sh/ruff/issues/7948).
-                // ```python
-                // class ModuleBrowser:
-                //     """Browse module classes and functions in IDLE."""
-                //     # ^ Insert a newline above here
-                //
-                //     def __init__(self, master, path, *, _htest=False, _utest=False):
-                //         pass
-                // ```
-                if let Some(own_line) = node_comments
-                    .trailing
-                    .iter()
-                    .find(|comment| comment.line_position().is_own_line())
-                {
-                    if lines_before(own_line.start(), f.context().source()) < 2 {
-                        empty_line().fmt(f)?;
-                    }
+        if self.suite_kind == SuiteKind::Class {
+            // Comments after class docstrings need a newline between the docstring and the
+            // comment (https://github.com/astral-sh/ruff/issues/7948).
+            // ```python
+            // class ModuleBrowser:
+            //     """Browse module classes and functions in IDLE."""
+            //     # ^ Insert a newline above here
+            //
+            //     def __init__(self, master, path, *, _htest=False, _utest=False):
+            //         pass
+            // ```
+            if let Some(own_line) = node_comments
+                .trailing
+                .iter()
+                .find(|comment| comment.line_position().is_own_line())
+            {
+                if lines_before(own_line.start(), f.context().source()) < 2 {
+                    empty_line().fmt(f)?;
                 }
             }
-
-            trailing_comments(node_comments.trailing).fmt(f)
         }
+
+        trailing_comments(node_comments.trailing).fmt(f)
     }
 }
 
@@ -916,6 +955,58 @@ impl Format<PyFormatContext<'_>> for SuiteChildStatement<'_> {
     }
 }
 
+pub(crate) fn skip_range(
+    first: &Stmt,
+    statements: &[Stmt],
+    context: &PyFormatContext,
+) -> Option<TextRange> {
+    let start = first.start();
+    let mut last_statement = first;
+
+    let comments = context.comments();
+    let source = context.source();
+
+    for statement in statements {
+        if new_logical_line_between_statements(
+            source,
+            TextRange::new(last_statement.end(), statement.start()),
+        ) {
+            break;
+        }
+        last_statement = statement;
+    }
+
+    if has_skip_comment(comments.trailing(last_statement), source) {
+        Some(TextRange::new(
+            start,
+            trailing_semicolon(last_statement.into(), source)
+                .map_or_else(|| last_statement.end(), ruff_text_size::TextRange::end),
+        ))
+    } else {
+        None
+    }
+}
+
+fn new_logical_line_between_statements(source: &str, between_statement_range: TextRange) -> bool {
+    let mut tokenizer = SimpleTokenizer::new(source, between_statement_range).map(|tok| tok.kind());
+
+    while let Some(token) = tokenizer.next() {
+        match token {
+            SimpleTokenKind::Continuation => {
+                tokenizer.next();
+            }
+            SimpleTokenKind::Newline => {
+                return true;
+            }
+            // Since we are between statements, there are
+            // no non-trivia tokens, so there is no need to check
+            // for these and do an early return.
+            _ => {}
+        }
+    }
+    false
+}
+
 #[cfg(test)]
 mod tests {
     use ruff_formatter::format;
