@@ -5,8 +5,8 @@ use ruff_diagnostics::Edit;
 use ruff_python_ast::find_node::covering_node;
 use ruff_text_size::TextRange;
 use ty_project::Db;
-use ty_python_semantic::create_suppression_fix;
 use ty_python_semantic::lint::LintId;
+use ty_python_semantic::suppress_single;
 use ty_python_semantic::types::{UNDEFINED_REVEAL, UNRESOLVED_REFERENCE};
 
 /// A `QuickFix` Code Action
@@ -42,7 +42,7 @@ pub fn code_actions(
     // Suggest just suppressing the lint (always a valid option, but never ideal)
     actions.push(QuickFix {
         title: format!("Ignore '{}' for this line", lint_id.name()),
-        edits: create_suppression_fix(db, file, lint_id, diagnostic_range).into_edits(),
+        edits: suppress_single(db, file, lint_id, diagnostic_range).into_edits(),
         preferred: false,
     });
 
@@ -96,7 +96,7 @@ mod tests {
     fn add_ignore() {
         let test = CodeActionTest::with_source(r#"b = <START>a<END> / 10"#);
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:1:5
           |
@@ -112,7 +112,7 @@ mod tests {
     fn add_ignore_existing_comment() {
         let test = CodeActionTest::with_source(r#"b = <START>a<END> / 10  # fmt: off"#);
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:1:5
           |
@@ -128,7 +128,7 @@ mod tests {
     fn add_ignore_trailing_whitespace() {
         let test = CodeActionTest::with_source(r#"b = <START>a<END> / 10  "#);
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:1:5
           |
@@ -148,7 +148,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:2:5
           |
@@ -169,7 +169,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:2:5
           |
@@ -190,7 +190,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:2:5
           |
@@ -211,7 +211,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:2:5
           |
@@ -236,7 +236,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:3:9
           |
@@ -269,7 +269,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:3:9
           |
@@ -301,7 +301,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: Ignore 'unresolved-reference' for this line
          --> main.py:3:9
           |
@@ -437,6 +437,38 @@ mod tests {
         "#);
     }
 
+    #[test]
+    fn add_ignore_line_continuation_empty_lines() {
+        let test = CodeActionTest::with_source(
+            r#"b = bbbbb \
+    [  ccc # test
+
+        + <START>ddd<END>  \
+
+    ] # test
+        "#,
+        );
+
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        info[code-action]: Ignore 'unresolved-reference' for this line
+         --> main.py:4:11
+          |
+        2 |     [  ccc # test
+        3 |
+        4 |         + ddd  \
+          |           ^^^
+        5 |
+        6 |     ] # test
+          |
+        2 |     [  ccc # test
+        3 | 
+        4 |         + ddd  \
+          - 
+        5 +   # ty:ignore[unresolved-reference]
+        6 |     ] # test
+        ");
+    }
+
     #[test]
     fn undefined_reveal_type() {
         let test = CodeActionTest::with_source(
@@ -445,7 +477,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNDEFINED_REVEAL), @r"
+        assert_snapshot!(test.code_actions(&UNDEFINED_REVEAL), @"
         info[code-action]: import typing.reveal_type
          --> main.py:2:1
           |
@@ -577,7 +609,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: import importlib.abc.ExecutionLoader
          --> main.py:2:1
           |
@@ -614,7 +646,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: import importlib.abc.ExecutionLoader
          --> main.py:3:1
           |
@@ -652,7 +684,7 @@ mod tests {
         "#,
         );
 
-        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @r"
+        assert_snapshot!(test.code_actions(&UNRESOLVED_REFERENCE), @"
         info[code-action]: import importlib.abc.ExecutionLoader
          --> main.py:3:1
           |
