@@ -184,6 +184,7 @@ impl MemberExpr {
                     let start_offset = path.text_len();
 
                     match &*subscript.slice {
+                        // Handle integer subscripts, like `x[0]`.
                         ast::Expr::NumberLiteral(ast::ExprNumberLiteral {
                             value: ast::Number::Int(index),
                             ..
@@ -192,11 +193,61 @@ impl MemberExpr {
                             segments
                                 .push(SegmentInfo::new(SegmentKind::IntSubscript, start_offset));
                         }
+                        // Handle negative integer subscripts, like `x[-1]`.
+                        ast::Expr::UnaryOp(ast::ExprUnaryOp {
+                            op: ast::UnaryOp::USub,
+                            operand,
+                            ..
+                        }) => match operand.as_ref() {
+                            ast::Expr::NumberLiteral(ast::ExprNumberLiteral {
+                                value: ast::Number::Int(index),
+                                ..
+                            }) => {
+                                let _ = write!(path, "-{index}");
+                                segments.push(SegmentInfo::new(
+                                    SegmentKind::IntSubscript,
+                                    start_offset,
+                                ));
+                            }
+                            _ => return None,
+                        },
+                        // Handle positive integer subscripts with explicit plus, like `x[+1]`.
+                        ast::Expr::UnaryOp(ast::ExprUnaryOp {
+                            op: ast::UnaryOp::UAdd,
+                            operand,
+                            ..
+                        }) => match operand.as_ref() {
+                            ast::Expr::NumberLiteral(ast::ExprNumberLiteral {
+                                value: ast::Number::Int(index),
+                                ..
+                            }) => {
+                                let _ = write!(path, "{index}");
+                                segments.push(SegmentInfo::new(
+                                    SegmentKind::IntSubscript,
+                                    start_offset,
+                                ));
+                            }
+                            _ => return None,
+                        },
+                        // Handle boolean subscripts, like `x[True]` or `x[False]`.
+                        // In Python, `True` and `False` are equivalent to `1` and `0` for indexing.
+                        ast::Expr::BooleanLiteral(ast::ExprBooleanLiteral { value, .. }) => {
+                            let _ = write!(path, "{}", u8::from(*value));
+                            segments
+                                .push(SegmentInfo::new(SegmentKind::IntSubscript, start_offset));
+                        }
                         ast::Expr::StringLiteral(string) => {
                             let _ = write!(path, "{}", string.value);
                             segments
                                 .push(SegmentInfo::new(SegmentKind::StringSubscript, start_offset));
                         }
+                        // Handle bytes literal subscripts, like `x[b"key"]`.
+                        ast::Expr::BytesLiteral(bytes) => {
+                            let bytes_vec: Vec<u8> = bytes.value.bytes().collect();
+                            let _ = write!(path, "{}", String::from_utf8_lossy(&bytes_vec));
+                            segments
+                                .push(SegmentInfo::new(SegmentKind::BytesSubscript, start_offset));
+                        }
                         _ => {
                             return None;
                         }
@@ -260,6 +311,7 @@ impl std::fmt::Display for MemberExpr {
                 SegmentKind::Attribute => write!(f, ".{}", segment.text)?,
                 SegmentKind::IntSubscript => write!(f, "[{}]", segment.text)?,
                 SegmentKind::StringSubscript => write!(f, "[\"{}\"]", segment.text)?,
+                SegmentKind::BytesSubscript => write!(f, "[b\"{}\"]", segment.text)?,
             }
         }
 
@@ -562,6 +614,7 @@ impl SegmentInfo {
             0 => SegmentKind::Attribute,
             1 => SegmentKind::IntSubscript,
             2 => SegmentKind::StringSubscript,
+            3 => SegmentKind::BytesSubscript,
             _ => panic!("Invalid SegmentKind bits"),
         }
     }
@@ -591,6 +644,7 @@ enum SegmentKind {
     Attribute = 0,
     IntSubscript = 1,
     StringSubscript = 2,
+    BytesSubscript = 3,
 }
 
 /// Iterator over segments that converts `SegmentInfo` to `Segment` with text slices.
@@ -778,6 +832,7 @@ impl Iterator for SmallSegmentsInfoIterator {
             0 => SegmentKind::Attribute,
             1 => SegmentKind::IntSubscript,
             2 => SegmentKind::StringSubscript,
+            3 => SegmentKind::BytesSubscript,
             _ => panic!("Invalid SegmentKind bits"),
         };
 
