@@ -2,7 +2,7 @@ use std::ops::Range;
 
 use ruff_db::{files::File, parsed::ParsedModuleRef};
 use ruff_index::newtype_index;
-use ruff_python_ast as ast;
+use ruff_python_ast::{self as ast, NodeIndex};
 
 use crate::{
     Db,
@@ -35,6 +35,16 @@ impl<'db> ScopeId<'db> {
         self.scope(db).node()
     }
 
+    /// Returns `true` if this scope may require type context from its parent scope.
+    pub(crate) fn accepts_type_context(self, db: &dyn Db) -> bool {
+        matches!(
+            self.node(db),
+            NodeWithScopeKind::ListComprehension(_)
+                | NodeWithScopeKind::SetComprehension(_)
+                | NodeWithScopeKind::DictComprehension(_)
+        )
+    }
+
     pub(crate) fn scope(self, db: &dyn Db) -> &Scope {
         semantic_index(db, self.file(db)).scope(self.file_scope_id(db))
     }
@@ -463,6 +473,27 @@ impl NodeWithScopeKind {
             _ => None,
         }
     }
+
+    /// Returns the anchor node index for this scope, or `None` for the module scope.
+    ///
+    /// This is used to compute relative node indices for expressions within the scope,
+    /// providing a stable anchor that only changes when the scope-introducing node changes.
+    pub(crate) fn node_index(&self) -> Option<NodeIndex> {
+        match self {
+            Self::Module => None,
+            Self::Class(class) => Some(class.index()),
+            Self::ClassTypeParameters(class) => Some(class.index()),
+            Self::Function(function) => Some(function.index()),
+            Self::FunctionTypeParameters(function) => Some(function.index()),
+            Self::TypeAlias(type_alias) => Some(type_alias.index()),
+            Self::TypeAliasTypeParameters(type_alias) => Some(type_alias.index()),
+            Self::Lambda(lambda) => Some(lambda.index()),
+            Self::ListComprehension(comp) => Some(comp.index()),
+            Self::SetComprehension(comp) => Some(comp.index()),
+            Self::DictComprehension(comp) => Some(comp.index()),
+            Self::GeneratorExpression(generator) => Some(generator.index()),
+        }
+    }
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, get_size2::GetSize)]
