@@ -5,15 +5,17 @@ use ruff_python_ast::{Expr, ExprRef, HasNodeIndex, name::Name};
 use ruff_python_parser::Parsed;
 use ruff_source_file::LineIndex;
 use rustc_hash::FxHashMap;
+use ty_module_resolver::{
+    KnownModule, Module, ModuleName, list_modules, resolve_module, resolve_real_shadowable_module,
+};
 
-use crate::module_name::ModuleName;
-use crate::module_resolver::{KnownModule, Module, list_modules, resolve_module};
+use crate::Db;
+use crate::place::implicit_globals::all_implicit_module_globals;
 use crate::semantic_index::definition::Definition;
 use crate::semantic_index::scope::FileScopeId;
 use crate::semantic_index::semantic_index;
 use crate::types::list_members::{Member, all_members, all_reachable_members};
-use crate::types::{Type, binding_type, infer_scope_types};
-use crate::{Db, resolve_real_shadowable_module};
+use crate::types::{Type, binding_type, infer_complete_scope_types};
 
 /// The primary interface the LSP should use for querying semantic information about a [`File`].
 ///
@@ -105,7 +107,7 @@ impl<'db> SemanticModel<'db> {
 
     /// Returns completions for symbols available in a `import <CURSOR>` context.
     pub fn import_completions(&self) -> Vec<Completion<'db>> {
-        let typing_extensions = ModuleName::new("typing_extensions").unwrap();
+        let typing_extensions = ModuleName::new_static("typing_extensions").unwrap();
         let is_typing_extensions_available = self.file.is_stub(self.db)
             || resolve_real_shadowable_module(self.db, self.file, &typing_extensions).is_some();
         list_modules(self.db)
@@ -233,9 +235,27 @@ impl<'db> SemanticModel<'db> {
                 ),
             );
         }
+
+        // Add implicit module globals (like `__file__`, `__name__`, etc.) with their
+        // correct types. These are added before builtins so that the deduplication
+        // keeps the correct types (e.g., `__file__` is `str` for the current module,
+        // not `str | None`).
+        completions.extend(
+            all_implicit_module_globals(self.db).map(|(name, ty)| Completion {
+                name,
+                ty: Some(ty),
+                builtin: true,
+            }),
+        );
+
         // Builtins are available in all scopes.
-        let builtins = ModuleName::new("builtins").expect("valid module name");
+        let builtins = ModuleName::new_static("builtins").expect("valid module name");
         completions.extend(self.module_completions(&builtins));
+
+        // The above can sometimes result in duplicates. Get rid of them.
+        completions.sort_by(|c1, c2| c1.name.cmp(&c2.name));
+        completions.dedup_by(|c1, c2| c1.name == c2.name);
+
         completions
     }
 
@@ -343,7 +363,7 @@ impl<'db> SemanticModel<'db> {
         let index = semantic_index(self.db, self.file);
         let file_scope = index.expression_scope_id(&expr);
         let scope = file_scope.to_scope_id(self.db, self.file);
-        if !infer_scope_types(self.db, scope).is_string_annotation(expr) {
+        if !infer_complete_scope_types(self.db, scope).is_string_annotation(expr) {
             return None;
         }
 
@@ -380,7 +400,7 @@ pub struct MemberDefinition<'db> {
 /// single-underscore names. This matches the order of the variants defined for
 /// this enum, which is in turn picked up by the derived trait implementation
 /// for `Ord`.
-#[derive(Clone, Copy, Eq, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub enum NameKind {
     Normal,
     Dunder,
@@ -426,12 +446,6 @@ pub struct Completion<'db> {
     pub builtin: bool,
 }
 
-impl<'db> Completion<'db> {
-    pub fn is_type_check_only(&self, db: &'db dyn Db) -> bool {
-        self.ty.is_some_and(|ty| ty.is_type_check_only(db))
-    }
-}
-
 pub trait HasType {
     /// Returns the inferred type of `self`.
     ///
@@ -458,7 +472,7 @@ impl HasType for ast::ExprRef<'_> {
         let file_scope = index.try_expression_scope_id(&model.expr_ref_in_ast(*self))?;
         let scope = file_scope.to_scope_id(model.db, model.file);
 
-        infer_scope_types(model.db, scope).try_expression_type(*self)
+        infer_complete_scope_types(model.db, scope).try_expression_type(*self)
     }
 }
 
