@@ -53,50 +53,17 @@ use crate::types::function::FunctionType;
 use crate::types::generics::Specialization;
 use crate::types::unpacker::{UnpackResult, Unpacker};
 use crate::types::{
-    ClassLiteral, KnownClass, Truthiness, Type, TypeAndQualifiers, declaration_type,
+    ClassLiteral, KnownClass, StaticClassLiteral, Truthiness, Type, TypeAndQualifiers,
+    declaration_type,
 };
 use crate::unpack::Unpack;
 use builder::TypeInferenceBuilder;
+pub(super) use builder::UnsupportedComparisonError;
 
 mod builder;
 #[cfg(test)]
 mod tests;
 
-/// Infer all types for a [`ScopeId`], including all definitions and expressions in that scope.
-/// Use when checking a scope, or needing to provide a type for an arbitrary expression in the
-/// scope.
-#[salsa::tracked(returns(ref), cycle_fn=scope_cycle_recover, cycle_initial=scope_cycle_initial, heap_size=ruff_memory_usage::heap_size)]
-pub(crate) fn infer_scope_types<'db>(db: &'db dyn Db, scope: ScopeId<'db>) -> ScopeInference<'db> {
-    let file = scope.file(db);
-    let _span = tracing::trace_span!("infer_scope_types", scope=?scope.as_id(), ?file).entered();
-
-    let module = parsed_module(db, file).load(db);
-
-    // Using the index here is fine because the code below depends on the AST anyway.
-    // The isolation of the query is by the return inferred types.
-    let index = semantic_index(db, file);
-
-    TypeInferenceBuilder::new(db, InferenceRegion::Scope(scope), index, &module).finish_scope()
-}
-
-fn scope_cycle_recover<'db>(
-    db: &'db dyn Db,
-    cycle: &salsa::Cycle,
-    previous_inference: &ScopeInference<'db>,
-    inference: ScopeInference<'db>,
-    _scope: ScopeId<'db>,
-) -> ScopeInference<'db> {
-    inference.cycle_normalized(db, previous_inference, cycle)
-}
-
-fn scope_cycle_initial<'db>(
-    _db: &'db dyn Db,
-    id: salsa::Id,
-    _scope: ScopeId<'db>,
-) -> ScopeInference<'db> {
-    ScopeInference::cycle_initial(Type::divergent(id))
-}
-
 /// Infer all types for a [`Definition`] (including sub-expressions).
 /// Use when resolving a place use or public type of a place.
 #[salsa::tracked(returns(ref), cycle_fn=definition_cycle_recover, cycle_initial=definition_cycle_initial, heap_size=ruff_memory_usage::heap_size)]
@@ -180,6 +147,84 @@ fn deferred_cycle_initial<'db>(
     DefinitionInference::cycle_initial(definition.scope(db), Type::divergent(id))
 }
 
+/// Infer all types for a [`ScopeId`], including all definitions and expressions in that scope.
+/// Use when checking a scope, or needing to provide a type for an arbitrary expression in the
+/// scope.
+///
+/// Unlike [`infer_scope_types`], this function does not take a type context, as it may infer
+/// the parent scope to obtain the necessary type context by which to infer the inner scope.
+pub(crate) fn infer_complete_scope_types<'db>(
+    db: &'db dyn Db,
+    scope: ScopeId<'db>,
+) -> &'db ScopeInference<'db> {
+    // Scopes that may require type context are inferred during the inference of
+    // their outer scope.
+    if scope.accepts_type_context(db) {
+        let file = scope.file(db);
+        let index = semantic_index(db, file);
+
+        if let Some(parent_scope) = index.parent_scope_id(scope.file_scope_id(db)) {
+            // Note that nested lambdas or comprehensions may require recursing until we reach
+            // an outer scope that is independent of any type context.
+            return infer_complete_scope_types(db, parent_scope.to_scope_id(db, file));
+        }
+    }
+
+    infer_scope_types_impl(db, InferScope::new(db, scope, TypeContext::default()))
+}
+
+/// Infer all types for a [`ScopeId`], including all definitions and expressions in that scope.
+/// Use when checking a scope, or needing to provide a type for an arbitrary expression in the
+/// scope.
+///
+/// Note that you should generally use [`infer_complete_scope_types`] instead of this method,
+/// unless you have already obtained the necessary type context while inferring the parent scope.
+/// Inferring a nested scope independently without type context can lead to incorrect inferred
+/// types or diagnostics.
+pub(crate) fn infer_scope_types<'db>(
+    db: &'db dyn Db,
+    scope: ScopeId<'db>,
+    tcx: TypeContext<'db>,
+) -> &'db ScopeInference<'db> {
+    infer_scope_types_impl(db, InferScope::new(db, scope, tcx))
+}
+
+#[salsa::tracked(returns(ref), cycle_fn=scope_cycle_recover, cycle_initial=scope_cycle_initial, heap_size=ruff_memory_usage::heap_size)]
+pub(crate) fn infer_scope_types_impl<'db>(
+    db: &'db dyn Db,
+    input: InferScope<'db>,
+) -> ScopeInference<'db> {
+    let (scope, tcx) = input.into_inner(db);
+    let file = scope.file(db);
+    let _span = tracing::trace_span!("infer_scope_types", scope=?scope.as_id(), ?file).entered();
+
+    let module = parsed_module(db, file).load(db);
+
+    // Using the index here is fine because the code below depends on the AST anyway.
+    // The isolation of the query is by the return inferred types.
+    let index = semantic_index(db, file);
+
+    TypeInferenceBuilder::new(db, InferenceRegion::Scope(scope, tcx), index, &module).finish_scope()
+}
+
+fn scope_cycle_recover<'db>(
+    db: &'db dyn Db,
+    cycle: &salsa::Cycle,
+    previous_inference: &ScopeInference<'db>,
+    inference: ScopeInference<'db>,
+    _input: InferScope<'db>,
+) -> ScopeInference<'db> {
+    inference.cycle_normalized(db, previous_inference, cycle)
+}
+
+fn scope_cycle_initial<'db>(
+    _db: &'db dyn Db,
+    id: salsa::Id,
+    _input: InferScope<'db>,
+) -> ScopeInference<'db> {
+    ScopeInference::cycle_initial(Type::divergent(id))
+}
+
 /// Infer all types for an [`Expression`] (including sub-expressions).
 /// Use rarely; only for cases where we'd otherwise risk double-inferring an expression: RHS of an
 /// assignment, which might be unpacking/multi-target and thus part of multiple definitions, or a
@@ -197,7 +242,7 @@ pub(super) fn infer_expression_types_impl<'db>(
     db: &'db dyn Db,
     input: InferExpression<'db>,
 ) -> ExpressionInference<'db> {
-    let (expression, tcx) = (input.expression(db), input.tcx(db));
+    let (expression, tcx) = input.into_inner(db);
 
     let file = expression.file(db);
     let module = parsed_module(db, file).load(db);
@@ -235,8 +280,9 @@ fn expression_cycle_initial<'db>(
     id: salsa::Id,
     input: InferExpression<'db>,
 ) -> ExpressionInference<'db> {
+    let (expression, _) = input.into_inner(db);
     let cycle_recovery = Type::divergent(id);
-    ExpressionInference::cycle_initial(input.expression(db).scope(db), cycle_recovery)
+    ExpressionInference::cycle_initial(expression.scope(db), cycle_recovery)
 }
 
 /// Infers the type of an `expression` that is guaranteed to be in the same file as the calling query.
@@ -271,13 +317,15 @@ pub(crate) fn infer_expression_type<'db>(
 
 #[salsa::tracked(cycle_fn=single_expression_cycle_recover, cycle_initial=single_expression_cycle_initial, heap_size=ruff_memory_usage::heap_size)]
 fn infer_expression_type_impl<'db>(db: &'db dyn Db, input: InferExpression<'db>) -> Type<'db> {
-    let file = input.expression(db).file(db);
+    let (expression, _) = input.into_inner(db);
+
+    let file = expression.file(db);
     let module = parsed_module(db, file).load(db);
 
     // It's okay to call the "same file" version here because we're inside a salsa query.
     let inference = infer_expression_types_impl(db, input);
 
-    inference.expression_type(input.expression(db).node_ref(db, &module))
+    inference.expression_type(expression.node_ref(db, &module))
 }
 
 fn single_expression_cycle_recover<'db>(
@@ -308,6 +356,12 @@ pub(super) enum InferExpression<'db> {
     WithContext(ExpressionWithContext<'db>),
 }
 
+#[salsa::interned(debug, heap_size=ruff_memory_usage::heap_size)]
+pub(super) struct ExpressionWithContext<'db> {
+    expression: Expression<'db>,
+    tcx: TypeContext<'db>,
+}
+
 impl<'db> InferExpression<'db> {
     pub(super) fn new(
         db: &'db dyn Db,
@@ -317,43 +371,63 @@ impl<'db> InferExpression<'db> {
         if tcx.annotation.is_some() {
             InferExpression::WithContext(ExpressionWithContext::new(db, expression, tcx))
         } else {
-            // Drop the empty `TypeContext` to avoid the interning cost.
             InferExpression::Bare(expression)
         }
     }
 
-    fn expression(self, db: &'db dyn Db) -> Expression<'db> {
+    fn into_inner(self, db: &'db dyn Db) -> (Expression<'db>, TypeContext<'db>) {
         match self {
-            InferExpression::Bare(expression) => expression,
-            InferExpression::WithContext(expression_with_context) => {
-                expression_with_context.expression(db)
-            }
-        }
-    }
-
-    fn tcx(self, db: &'db dyn Db) -> TypeContext<'db> {
-        match self {
-            InferExpression::Bare(_) => TypeContext::default(),
-            InferExpression::WithContext(expression_with_context) => {
-                expression_with_context.tcx(db)
-            }
+            InferExpression::Bare(expression) => (expression, TypeContext::default()),
+            InferExpression::WithContext(expression_with_context) => (
+                expression_with_context.expression(db),
+                expression_with_context.tcx(db),
+            ),
         }
     }
 }
 
-/// An `Expression` with a `TypeContext`.
+/// A `ScopeId` with an optional `TypeContext`.
+#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq, salsa::Supertype, salsa::Update)]
+pub(super) enum InferScope<'db> {
+    Bare(ScopeId<'db>),
+    WithContext(ScopeWithContext<'db>),
+}
+
 #[salsa::interned(debug, heap_size=ruff_memory_usage::heap_size)]
-pub(super) struct ExpressionWithContext<'db> {
-    expression: Expression<'db>,
+pub(super) struct ScopeWithContext<'db> {
+    scope: ScopeId<'db>,
     tcx: TypeContext<'db>,
 }
 
+impl<'db> InferScope<'db> {
+    pub(super) fn new(
+        db: &'db dyn Db,
+        scope: ScopeId<'db>,
+        tcx: TypeContext<'db>,
+    ) -> InferScope<'db> {
+        if tcx.annotation.is_some() {
+            InferScope::WithContext(ScopeWithContext::new(db, scope, tcx))
+        } else {
+            InferScope::Bare(scope)
+        }
+    }
+
+    fn into_inner(self, db: &'db dyn Db) -> (ScopeId<'db>, TypeContext<'db>) {
+        match self {
+            InferScope::Bare(scope) => (scope, TypeContext::default()),
+            InferScope::WithContext(scope_with_context) => {
+                (scope_with_context.scope(db), scope_with_context.tcx(db))
+            }
+        }
+    }
+}
+
 /// The type context for a given expression, namely the type annotation
 /// in an annotated assignment.
 ///
 /// Knowing the outer type context when inferring an expression can enable
 /// more precise inference results, aka "bidirectional type inference".
-#[derive(Default, Copy, Clone, Debug, PartialEq, Eq, Hash, get_size2::GetSize)]
+#[derive(Default, Copy, Clone, Debug, PartialEq, Eq, Hash, get_size2::GetSize, salsa::Update)]
 pub(crate) struct TypeContext<'db> {
     pub(crate) annotation: Option<Type<'db>>,
 }
@@ -464,7 +538,7 @@ pub(crate) fn nearest_enclosing_class<'db>(
     db: &'db dyn Db,
     semantic: &SemanticIndex<'db>,
     scope: ScopeId,
-) -> Option<ClassLiteral<'db>> {
+) -> Option<StaticClassLiteral<'db>> {
     semantic
         .ancestor_scopes(scope.file_scope_id(db))
         .find_map(|(_, ancestor_scope)| {
@@ -473,6 +547,7 @@ pub(crate) fn nearest_enclosing_class<'db>(
             declaration_type(db, definition)
                 .inner_type()
                 .as_class_literal()
+                .and_then(ClassLiteral::as_static)
         })
 }
 
@@ -510,7 +585,7 @@ pub(crate) enum InferenceRegion<'db> {
     /// infer deferred types for a [`Definition`]
     Deferred(Definition<'db>),
     /// infer types for an entire [`ScopeId`]
-    Scope(ScopeId<'db>),
+    Scope(ScopeId<'db>, TypeContext<'db>),
 }
 
 impl<'db> InferenceRegion<'db> {
@@ -520,7 +595,7 @@ impl<'db> InferenceRegion<'db> {
             InferenceRegion::Definition(definition) | InferenceRegion::Deferred(definition) => {
                 definition.scope(db)
             }
-            InferenceRegion::Scope(scope) => scope,
+            InferenceRegion::Scope(scope, _) => scope,
         }
     }
 }
