@@ -1,19 +1,19 @@
+use super::TypeVarVariance;
 use crate::place::PlaceAndQualifiers;
 use crate::semantic_index::definition::Definition;
 use crate::types::constraints::ConstraintSet;
 use crate::types::generics::InferableTypeVars;
 use crate::types::protocol_class::ProtocolClass;
+use crate::types::relation::{HasRelationToVisitor, IsDisjointVisitor, TypeRelation};
 use crate::types::variance::VarianceInferable;
 use crate::types::{
-    ApplyTypeMappingVisitor, BoundTypeVarInstance, ClassType, DynamicType,
-    FindLegacyTypeVarsVisitor, HasRelationToVisitor, IsDisjointVisitor, KnownClass,
-    MaterializationKind, MemberLookupPolicy, NormalizedVisitor, SpecialFormType, Type, TypeContext,
-    TypeMapping, TypeRelation, TypeVarBoundOrConstraints, TypedDictType, todo_type,
+    ApplyTypeMappingVisitor, BoundTypeVarInstance, ClassLiteral, ClassType, DynamicClassLiteral,
+    DynamicType, FindLegacyTypeVarsVisitor, KnownClass, MaterializationKind, MemberLookupPolicy,
+    NormalizedVisitor, SpecialFormType, Type, TypeContext, TypeMapping, TypeVarBoundOrConstraints,
+    TypedDictType, UnionType, todo_type,
 };
 use crate::{Db, FxOrderSet};
 
-use super::TypeVarVariance;
-
 /// A type that represents `type[C]`, i.e. the class object `C` and class objects that are subclasses of `C`.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, salsa::Update, get_size2::GetSize)]
 pub struct SubclassOfType<'db> {
@@ -79,6 +79,18 @@ impl<'db> SubclassOfType<'db> {
 
     /// Given an instance of the class or type variable `T`, returns a [`Type`] instance representing `type[T]`.
     pub(crate) fn try_from_instance(db: &'db dyn Db, ty: Type<'db>) -> Option<Type<'db>> {
+        // Handle unions by distributing `type[]` over each element:
+        // `type[A | B]` -> `type[A] | type[B]`
+        if let Type::Union(union) = ty {
+            return UnionType::try_from_elements(
+                db,
+                union
+                    .elements(db)
+                    .iter()
+                    .map(|element| Self::try_from_instance(db, *element)),
+            );
+        }
+
         SubclassOfInner::try_from_instance(db, ty).map(|subclass_of| Self::from(db, subclass_of))
     }
 
@@ -290,10 +302,39 @@ impl<'db> SubclassOfType<'db> {
         }
     }
 
+    /// Compute the metatype of this `type[T]`.
+    ///
+    /// For `type[C]` where `C` is a concrete class, this returns `type[metaclass(C)]`.
+    /// For `type[T]` where `T` is a `TypeVar`, this computes the metatype based on the
+    /// `TypeVar`'s bounds or constraints.
+    pub(crate) fn to_meta_type(self, db: &'db dyn Db) -> Type<'db> {
+        match self.subclass_of.with_transposed_type_var(db) {
+            SubclassOfInner::Dynamic(dynamic) => {
+                SubclassOfType::from(db, SubclassOfInner::Dynamic(dynamic))
+            }
+            SubclassOfInner::Class(class) => SubclassOfType::try_from_type(db, class.metaclass(db))
+                .unwrap_or(SubclassOfType::subclass_of_unknown()),
+            // For `type[T]` where `T` is a TypeVar, `with_transposed_type_var` transforms
+            // the bounds from instance types to `type[]` types. For example, `type[T]` where
+            // `T: A | B` becomes a TypeVar with bound `type[A] | type[B]`. The metatype is
+            // then the metatype of that bound.
+            SubclassOfInner::TypeVar(bound_typevar) => {
+                match bound_typevar.typevar(db).bound_or_constraints(db) {
+                    // `with_transposed_type_var` always adds a bound for unbounded TypeVars
+                    None => unreachable!(),
+                    Some(TypeVarBoundOrConstraints::UpperBound(bound)) => bound.to_meta_type(db),
+                    Some(TypeVarBoundOrConstraints::Constraints(constraints)) => {
+                        constraints.as_type(db).to_meta_type(db)
+                    }
+                }
+            }
+        }
+    }
+
     pub(crate) fn is_typed_dict(self, db: &'db dyn Db) -> bool {
         self.subclass_of
             .into_class(db)
-            .is_some_and(|class| class.class_literal(db).0.is_typed_dict(db))
+            .is_some_and(|class| class.class_literal(db).is_typed_dict(db))
     }
 }
 
@@ -493,3 +534,9 @@ impl<'db> From<SubclassOfType<'db>> for Type<'db> {
         }
     }
 }
+
+impl<'db> From<DynamicClassLiteral<'db>> for SubclassOfInner<'db> {
+    fn from(value: DynamicClassLiteral<'db>) -> Self {
+        SubclassOfInner::Class(ClassType::NonGeneric(ClassLiteral::Dynamic(value)))
+    }
+}
