@@ -5,7 +5,8 @@ use salsa::plumbing::AsId;
 use crate::{db::Db, types::bound_super::SuperOwnerKind};
 
 use super::{
-    DynamicType, TodoType, Type, TypeIsType, class_base::ClassBase, subclass_of::SubclassOfInner,
+    DynamicType, TodoType, Type, TypeGuardLike, TypeGuardType, TypeIsType, class_base::ClassBase,
+    subclass_of::SubclassOfInner,
 };
 
 /// Return an [`Ordering`] that describes the canonical order in which two types should appear
@@ -132,6 +133,10 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
         (Type::TypeIs(_), _) => Ordering::Less,
         (_, Type::TypeIs(_)) => Ordering::Greater,
 
+        (Type::TypeGuard(left), Type::TypeGuard(right)) => typeguard_ordering(db, *left, *right),
+        (Type::TypeGuard(_), _) => Ordering::Less,
+        (_, Type::TypeGuard(_)) => Ordering::Greater,
+
         (Type::NominalInstance(left), Type::NominalInstance(right)) => {
             left.class(db).cmp(&right.class(db))
         }
@@ -184,6 +189,17 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
                 }
                 (SuperOwnerKind::Instance(_), _) => Ordering::Less,
                 (_, SuperOwnerKind::Instance(_)) => Ordering::Greater,
+                (
+                    SuperOwnerKind::InstanceTypeVar(left, _),
+                    SuperOwnerKind::InstanceTypeVar(right, _),
+                ) => left.cmp(&right),
+                (SuperOwnerKind::InstanceTypeVar(..), _) => Ordering::Less,
+                (_, SuperOwnerKind::InstanceTypeVar(..)) => Ordering::Greater,
+                (SuperOwnerKind::ClassTypeVar(left, _), SuperOwnerKind::ClassTypeVar(right, _)) => {
+                    left.cmp(&right)
+                }
+                (SuperOwnerKind::ClassTypeVar(..), _) => Ordering::Less,
+                (_, SuperOwnerKind::ClassTypeVar(..)) => Ordering::Greater,
                 (SuperOwnerKind::Dynamic(left), SuperOwnerKind::Dynamic(right)) => {
                     dynamic_elements_ordering(left, right)
                 }
@@ -212,9 +228,7 @@ pub(super) fn union_or_intersection_elements_ordering<'db>(
         (Type::TypeAlias(_), _) => Ordering::Less,
         (_, Type::TypeAlias(_)) => Ordering::Greater,
 
-        (Type::TypedDict(left), Type::TypedDict(right)) => {
-            left.defining_class().cmp(&right.defining_class())
-        }
+        (Type::TypedDict(left), Type::TypedDict(right)) => left.cmp(right),
         (Type::TypedDict(_), _) => Ordering::Less,
         (_, Type::TypedDict(_)) => Ordering::Greater,
 
@@ -286,13 +300,13 @@ fn dynamic_elements_ordering(left: DynamicType, right: DynamicType) -> Ordering
     }
 }
 
-/// Determine a canonical order for two instances of [`TypeIsType`].
+/// Generic helper for ordering type guard-like types.
 ///
 /// The following criteria are considered, in order:
 /// * Boundness: Unbound precedes bound
 /// * Symbol name: String comparison
 /// * Guarded type: [`union_or_intersection_elements_ordering`]
-fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering {
+fn guard_like_ordering<'db, T: TypeGuardLike<'db>>(db: &'db dyn Db, left: T, right: T) -> Ordering {
     let (left_ty, right_ty) = (left.return_type(db), right.return_type(db));
 
     match (left.place_info(db), right.place_info(db)) {
@@ -307,3 +321,13 @@ fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering
         },
     }
 }
+
+/// Determine a canonical order for two instances of [`TypeIsType`].
+fn typeis_ordering(db: &dyn Db, left: TypeIsType, right: TypeIsType) -> Ordering {
+    guard_like_ordering(db, left, right)
+}
+
+/// Determine a canonical order for two instances of [`TypeGuardType`].
+fn typeguard_ordering(db: &dyn Db, left: TypeGuardType, right: TypeGuardType) -> Ordering {
+    guard_like_ordering(db, left, right)
+}
