@@ -19,6 +19,7 @@ use super::diagnostic::{
 use super::{ApplyTypeMappingVisitor, Type, TypeMapping, visitor};
 use crate::Db;
 use crate::semantic_index::definition::Definition;
+use crate::types::TypeDefinition;
 use crate::types::class::FieldKind;
 use crate::types::constraints::{ConstraintSet, IteratorConstraintsExtension};
 use crate::types::generics::InferableTypeVars;
@@ -48,7 +49,14 @@ impl Default for TypedDictParams {
 
 /// Type that represents the set of all inhabitants (`dict` instances) that conform to
 /// a given `TypedDict` schema.
-#[derive(Debug, Copy, Clone, PartialEq, Eq, salsa::Update, Hash, get_size2::GetSize)]
+///
+/// # Ordering
+/// Ordering is derived from the variant order (`Class` < `Synthesized`) and the inner types.
+/// The Salsa IDs of inner types may change between runs or when the type was garbage collected
+/// and recreated.
+#[derive(
+    Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, salsa::Update, Hash, get_size2::GetSize,
+)]
 pub enum TypedDictType<'db> {
     /// A reference to the class (inheriting from `typing.TypedDict`) that specifies the
     /// schema of this `TypedDict`.
@@ -73,7 +81,9 @@ impl<'db> TypedDictType<'db> {
     pub(crate) fn items(self, db: &'db dyn Db) -> &'db TypedDictSchema<'db> {
         #[salsa::tracked(returns(ref), heap_size=ruff_memory_usage::heap_size)]
         fn class_based_items<'db>(db: &'db dyn Db, class: ClassType<'db>) -> TypedDictSchema<'db> {
-            let (class_literal, specialization) = class.class_literal(db);
+            let Some((class_literal, specialization)) = class.static_class_literal(db) else {
+                return TypedDictSchema::default();
+            };
             class_literal
                 .fields(db, specialization, CodeGeneratorKind::TypedDict)
                 .into_iter()
@@ -293,7 +303,14 @@ impl<'db> TypedDictType<'db> {
 
     pub fn definition(self, db: &'db dyn Db) -> Option<Definition<'db>> {
         match self {
-            TypedDictType::Class(defining_class) => Some(defining_class.definition(db)),
+            TypedDictType::Class(defining_class) => defining_class.definition(db),
+            TypedDictType::Synthesized(_) => None,
+        }
+    }
+
+    pub fn type_definition(self, db: &'db dyn Db) -> Option<TypeDefinition<'db>> {
+        match self {
+            TypedDictType::Class(defining_class) => defining_class.type_definition(db),
             TypedDictType::Synthesized(_) => None,
         }
     }
@@ -878,7 +895,11 @@ pub(super) fn validate_typed_dict_dict_literal<'db>(
     }
 }
 
+/// # Ordering
+/// Ordering is based on the type's salsa-assigned id and not on its values.
+/// The id may change between runs, or when the type was garbage collected and recreated.
 #[salsa::interned(debug, heap_size=ruff_memory_usage::heap_size)]
+#[derive(PartialOrd, Ord)]
 pub struct SynthesizedTypedDictType<'db> {
     #[returns(ref)]
     pub(crate) items: TypedDictSchema<'db>,
