@@ -1,12 +1,12 @@
-use rustc_hash::FxHashMap;
+use rustc_hash::FxHashSet;
 
 use crate::{
-    Db, FxIndexSet,
+    Db,
     types::{
         BoundMethodType, BoundSuperType, BoundTypeVarInstance, CallableType, GenericAlias,
         IntersectionType, KnownBoundMethodType, KnownInstanceType, NominalInstanceType,
         PropertyInstanceType, ProtocolInstanceType, SubclassOfType, Type, TypeAliasType,
-        TypeIsType, TypeVarInstance, TypedDictType, UnionType,
+        TypeGuardType, TypeIsType, TypeVarInstance, TypedDictType, UnionType,
         bound_super::walk_bound_super_type,
         class::walk_generic_alias,
         function::{FunctionType, walk_function_type},
@@ -16,13 +16,10 @@ use crate::{
         walk_bound_method_type, walk_bound_type_var_type, walk_callable_type,
         walk_intersection_type, walk_known_instance_type, walk_method_wrapper_type,
         walk_property_instance_type, walk_type_alias_type, walk_type_var_type,
-        walk_typed_dict_type, walk_typeis_type, walk_union,
+        walk_typed_dict_type, walk_typeguard_type, walk_typeis_type, walk_union,
     },
 };
-use std::{
-    cell::{Cell, RefCell},
-    collections::hash_map::Entry,
-};
+use std::cell::{Cell, RefCell};
 
 /// A visitor trait that recurses into nested types.
 ///
@@ -55,6 +52,10 @@ pub(crate) trait TypeVisitor<'db> {
         walk_typeis_type(db, type_is, self);
     }
 
+    fn visit_typeguard_type(&self, db: &'db dyn Db, type_is: TypeGuardType<'db>) {
+        walk_typeguard_type(db, type_is, self);
+    }
+
     fn visit_subclass_of_type(&self, db: &'db dyn Db, subclass_of: SubclassOfType<'db>) {
         walk_subclass_of_type(db, subclass_of, self);
     }
@@ -132,6 +133,7 @@ pub(super) enum NonAtomicType<'db> {
     NominalInstance(NominalInstanceType<'db>),
     PropertyInstance(PropertyInstanceType<'db>),
     TypeIs(TypeIsType<'db>),
+    TypeGuard(TypeGuardType<'db>),
     TypeVar(BoundTypeVarInstance<'db>),
     ProtocolInstance(ProtocolInstanceType<'db>),
     TypedDict(TypedDictType<'db>),
@@ -200,6 +202,9 @@ impl<'db> From<Type<'db>> for TypeKind<'db> {
                 TypeKind::NonAtomic(NonAtomicType::TypeVar(bound_typevar))
             }
             Type::TypeIs(type_is) => TypeKind::NonAtomic(NonAtomicType::TypeIs(type_is)),
+            Type::TypeGuard(type_guard) => {
+                TypeKind::NonAtomic(NonAtomicType::TypeGuard(type_guard))
+            }
             Type::TypedDict(typed_dict) => {
                 TypeKind::NonAtomic(NonAtomicType::TypedDict(typed_dict))
             }
@@ -238,6 +243,7 @@ pub(super) fn walk_non_atomic_type<'db, V: TypeVisitor<'db> + ?Sized>(
             visitor.visit_property_instance_type(db, property);
         }
         NonAtomicType::TypeIs(type_is) => visitor.visit_typeis_type(db, type_is),
+        NonAtomicType::TypeGuard(type_guard) => visitor.visit_typeguard_type(db, type_guard),
         NonAtomicType::TypeVar(bound_typevar) => {
             visitor.visit_bound_type_var_type(db, bound_typevar);
         }
@@ -273,7 +279,7 @@ pub(crate) fn walk_type_with_recursion_guard<'db>(
 }
 
 #[derive(Default, Debug)]
-pub(crate) struct TypeCollector<'db>(RefCell<FxIndexSet<Type<'db>>>);
+pub(crate) struct TypeCollector<'db>(RefCell<FxHashSet<Type<'db>>>);
 
 impl<'db> TypeCollector<'db> {
     pub(crate) fn type_was_already_seen(&self, ty: Type<'db>) -> bool {
@@ -330,148 +336,3 @@ pub(super) fn any_over_type<'db>(
     visitor.visit_type(db, ty);
     visitor.found_matching_type.get()
 }
-
-/// Returns the maximum number of layers of generic specializations for a given type.
-///
-/// For example, `int` has a depth of `0`, `list[int]` has a depth of `1`, and `list[set[int]]`
-/// has a depth of `2`. A set-theoretic type like `list[int] | list[list[int]]` has a maximum
-/// depth of `2`.
-fn specialization_depth(db: &dyn Db, ty: Type<'_>) -> usize {
-    #[derive(Debug, Default)]
-    struct SpecializationDepthVisitor<'db> {
-        seen_types: RefCell<FxHashMap<NonAtomicType<'db>, Option<usize>>>,
-        max_depth: Cell<usize>,
-    }
-
-    impl<'db> TypeVisitor<'db> for SpecializationDepthVisitor<'db> {
-        fn should_visit_lazy_type_attributes(&self) -> bool {
-            false
-        }
-
-        fn visit_type(&self, db: &'db dyn Db, ty: Type<'db>) {
-            match TypeKind::from(ty) {
-                TypeKind::Atomic => {
-                    if ty.is_divergent() {
-                        self.max_depth.set(usize::MAX);
-                    }
-                }
-                TypeKind::NonAtomic(non_atomic_type) => {
-                    match self.seen_types.borrow_mut().entry(non_atomic_type) {
-                        Entry::Occupied(cached_depth) => {
-                            self.max_depth
-                                .update(|current| current.max(cached_depth.get().unwrap_or(0)));
-                            return;
-                        }
-                        Entry::Vacant(entry) => {
-                            entry.insert(None);
-                        }
-                    }
-
-                    let self_depth: usize =
-                        matches!(non_atomic_type, NonAtomicType::GenericAlias(_)).into();
-
-                    let previous_max_depth = self.max_depth.replace(0);
-                    walk_non_atomic_type(db, non_atomic_type, self);
-
-                    self.max_depth.update(|max_child_depth| {
-                        previous_max_depth.max(max_child_depth.saturating_add(self_depth))
-                    });
-
-                    self.seen_types
-                        .borrow_mut()
-                        .insert(non_atomic_type, Some(self.max_depth.get()));
-                }
-            }
-        }
-    }
-
-    let visitor = SpecializationDepthVisitor::default();
-    visitor.visit_type(db, ty);
-    visitor.max_depth.get()
-}
-
-pub(super) fn exceeds_max_specialization_depth(db: &dyn Db, ty: Type<'_>) -> bool {
-    // To prevent infinite recursion during type inference for infinite types, we fall back to
-    // `C[Divergent]` once a certain amount of levels of specialization have occurred. For
-    // example:
-    //
-    // ```py
-    // x = 1
-    // while random_bool():
-    //     x = [x]
-    //
-    // reveal_type(x)  # Unknown | Literal[1] | list[Divergent]
-    // ```
-    const MAX_SPECIALIZATION_DEPTH: usize = 10;
-
-    specialization_depth(db, ty) > MAX_SPECIALIZATION_DEPTH
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::{db::tests::setup_db, types::KnownClass};
-
-    #[test]
-    fn test_generics_layering_depth() {
-        let db = setup_db();
-
-        let int = || KnownClass::Int.to_instance(&db);
-        let list = |element| KnownClass::List.to_specialized_instance(&db, [element]);
-        let dict = |key, value| KnownClass::Dict.to_specialized_instance(&db, [key, value]);
-        let set = |element| KnownClass::Set.to_specialized_instance(&db, [element]);
-        let str = || KnownClass::Str.to_instance(&db);
-        let bytes = || KnownClass::Bytes.to_instance(&db);
-
-        let list_of_int = list(int());
-        assert_eq!(specialization_depth(&db, list_of_int), 1);
-
-        let list_of_list_of_int = list(list_of_int);
-        assert_eq!(specialization_depth(&db, list_of_list_of_int), 2);
-
-        let list_of_list_of_list_of_int = list(list_of_list_of_int);
-        assert_eq!(specialization_depth(&db, list_of_list_of_list_of_int), 3);
-
-        assert_eq!(specialization_depth(&db, set(dict(str(), list_of_int))), 3);
-
-        assert_eq!(
-            specialization_depth(
-                &db,
-                UnionType::from_elements(&db, [list_of_list_of_list_of_int, list_of_list_of_int])
-            ),
-            3
-        );
-
-        assert_eq!(
-            specialization_depth(
-                &db,
-                UnionType::from_elements(&db, [list_of_list_of_int, list_of_list_of_list_of_int])
-            ),
-            3
-        );
-
-        assert_eq!(
-            specialization_depth(
-                &db,
-                Type::heterogeneous_tuple(&db, [Type::heterogeneous_tuple(&db, [int()])])
-            ),
-            2
-        );
-
-        assert_eq!(
-            specialization_depth(&db, Type::heterogeneous_tuple(&db, [list_of_int, str()])),
-            2
-        );
-
-        assert_eq!(
-            specialization_depth(
-                &db,
-                list(UnionType::from_elements(
-                    &db,
-                    [list(int()), list(str()), list(bytes())]
-                ))
-            ),
-            2
-        );
-    }
-}
