@@ -200,13 +200,28 @@ impl UnmatchedWithColumn for &Diagnostic {
 
 /// Discard `@Todo`-type metadata from expected types, which is not available
 /// when running in release mode.
+///
+/// Some `@Todo` variants (like `@Todo(StarredExpression)` and `@Todo(typing.Unpack)`)
+/// are hardcoded enum variants that always display their message, so we preserve those.
 fn discard_todo_metadata(ty: &str) -> Cow<'_, str> {
     #[cfg(not(debug_assertions))]
     {
+        /// `@Todo` variants that are hardcoded and always display their message,
+        /// even in release mode.
+        const PRESERVED_TODO_VARIANTS: &[&str] =
+            &["@Todo(StarredExpression)", "@Todo(typing.Unpack)"];
+
         static TODO_METADATA_REGEX: LazyLock<regex::Regex> =
             LazyLock::new(|| regex::Regex::new(r"@Todo\([^)]*\)").unwrap());
 
-        TODO_METADATA_REGEX.replace_all(ty, "@Todo")
+        TODO_METADATA_REGEX.replace_all(ty, |caps: &regex::Captures| {
+            let matched = caps.get(0).unwrap().as_str();
+            if PRESERVED_TODO_VARIANTS.contains(&matched) {
+                matched.to_string()
+            } else {
+                "@Todo".to_string()
+            }
+        })
     }
 
     #[cfg(debug_assertions)]
@@ -216,11 +231,11 @@ fn discard_todo_metadata(ty: &str) -> Cow<'_, str> {
 /// Normalize paths in diagnostics to Unix paths before comparing them against
 /// the expected type. Doing otherwise means that it's hard to write cross-platform
 /// tests, since in some edge cases the display of a type can include a path to the
-/// file in which the type was defined (e.g. `foo.bar.A @ src/foo/bar.py:10` on Unix,
-/// but `foo.bar.A @ src\foo\bar.py:10` on Windows).
+/// file in which the type was defined (e.g. `foo.bar.A @ src/foo/bar.py:10:5` on Unix,
+/// but `foo.bar.A @ src\foo\bar.py:10:5` on Windows).
 fn normalize_paths(ty: &str) -> Cow<'_, str> {
     static PATH_IN_CLASS_DISPLAY_REGEX: LazyLock<regex::Regex> =
-        LazyLock::new(|| regex::Regex::new(r"( @ )(.+)(\.pyi?:\d)").unwrap());
+        LazyLock::new(|| regex::Regex::new(r"( @ )([^\.]+?)(\.pyi?:\d)").unwrap());
 
     fn normalize_path_captures(path_captures: &regex::Captures) -> String {
         let normalized_path = std::path::Path::new(&path_captures[2])
@@ -319,7 +334,7 @@ impl Matcher {
                         .column
                         .is_none_or(|col| col == self.column(diagnostic));
                     let message_matches = error.message_contains.is_none_or(|needle| {
-                        normalize_paths(&diagnostic.concise_message().to_string()).contains(needle)
+                        normalize_paths(&diagnostic.concise_message().to_str()).contains(needle)
                     });
                     lint_name_matches && column_matches && message_matches
                 });
@@ -345,6 +360,8 @@ impl Matcher {
                         return false;
                     };
 
+                    let primary_annotation = normalize_paths(primary_annotation);
+
                     // reveal_type, reveal_protocol_interface
                     if matches!(
                         primary_message,
@@ -403,9 +420,8 @@ mod tests {
     use ruff_python_trivia::textwrap::dedent;
     use ruff_source_file::OneIndexed;
     use ruff_text_size::TextRange;
-    use ty_python_semantic::{
-        Program, ProgramSettings, PythonPlatform, PythonVersionWithSource, SearchPathSettings,
-    };
+    use ty_module_resolver::SearchPathSettings;
+    use ty_python_semantic::{Program, ProgramSettings, PythonPlatform, PythonVersionWithSource};
 
     struct ExpectedDiagnostic {
         id: DiagnosticId,
@@ -1379,4 +1395,28 @@ mod tests {
             )],
         );
     }
+
+    #[test]
+    fn trailing_quote_without_message_not_allowed() {
+        let source = "x  # error: [some-rule]\"";
+        let result = get_result(
+            source,
+            vec![ExpectedDiagnostic::new(
+                DiagnosticId::lint("some-rule"),
+                "some message",
+                0,
+            )],
+        );
+
+        assert_fail(
+            result,
+            &[(
+                0,
+                &[
+                    "invalid assertion: expected message text and closing '\"' after opening '\"'",
+                    r#"unexpected error: 1 [some-rule] "some message""#,
+                ],
+            )],
+        );
+    }
 }
